from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from .models import Recipe, Tag, Rating, Setting, Order
from django.utils.html import format_html
from import_export import resources
from import_export.admin import ImportExportModelAdmin
from solo.admin import SingletonModelAdmin
from .forms import SettingForm


class RecipeResource(resources.ModelResource):
    class Meta:
        model = Recipe


class TagResource(resources.ModelResource):
    class Meta:
        model = Tag


class RatingResource(resources.ModelResource):
    class Meta:
        model = Rating


class OrderResource(resources.ModelResource):
    class Meta:
        model = Order


User = get_user_model()


@admin.register(Setting)
class SettingAdmin(SingletonModelAdmin):
    form = SettingForm


@admin.register(User)
class CustomUserAdmin(UserAdmin):
    fieldsets = UserAdmin.fieldsets + (
        (None, {"fields": ("avatar", "bio", "language", "favorites")}),
    )


@admin.register(Recipe)
class RecipeAdmin(ImportExportModelAdmin):
    resource_classes = [RecipeResource]
    list_display = (
        "title",
        "uploaded_by",
        "price",
        "created_at",
        "is_highlighted",
        "show_url",
    )
    list_editable = ("is_highlighted",)
    readonly_fields = ("created_at", "updated_at")

    def save_model(self, request, obj, form, change):
        # set uploaded_by automatically when creating in admin
        if not change and not obj.uploaded_by:
            obj.uploaded_by = request.user
        super().save_model(request, obj, form, change)

    def show_url(self, obj):
        url = obj.get_absolute_url()
        return format_html("<a href='{url}'>{url}</a>", url=url)

    show_url.short_description = "Recipe Link"  # ty:ignore[unresolved-attribute]


@admin.register(Tag)
class TagAdmin(ImportExportModelAdmin):
    resource_classes = [TagResource]


@admin.register(Rating)
class RatingAdmin(ImportExportModelAdmin):
    resource_classes = [RatingResource]


@admin.register(Order)
class OrderAdmin(ImportExportModelAdmin):
    resource_classes = [OrderResource]
    list_display = ("id", "user", "recipe", "status", "total_price", "created_at")
    list_filter = ("status", "created_at")
    search_fields = ("user__username", "recipe__title")
    readonly_fields = ("total_price", "created_at", "updated_at")



# Generated by Django 6.0 on 2026-01-14 09:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0003_setting"),
    ]

    operations = [
        migrations.AlterField(
            model_name="setting",
            name="ai_api_key",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="setting",
            name="ai_connection_point",
            field=models.URLField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="setting",
            name="ai_model",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="setting",
            name="email",
            field=models.EmailField(blank=True, max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name="setting",
            name="site_description",
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 6.0.1 on 2026-01-20 10:49

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0006_historicalrecipe_is_highlighted_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalrecipe",
            name="price",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="Price (€)",
            ),
        ),
        migrations.AddField(
            model_name="recipe",
            name="price",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=6,
                null=True,
                verbose_name="Price (€)",
            ),
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("PENDING", "Pending"),
                            ("COMPLETED", "Completed"),
                            ("CANCELLED", "Cancelled"),
                        ],
                        default="PENDING",
                        max_length=20,
                    ),
                ),
                ("total_price", models.DecimalField(decimal_places=2, max_digits=6)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "recipe",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="orders",
                        to="sandwitches.recipe",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="orders",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Order",
                "verbose_name_plural": "Orders",
                "ordering": ("-created_at",),
            },
        ),
    ]



from django.db import models
from django.utils.text import slugify
from .storage import HashedFilenameStorage
from simple_history.models import HistoricalRecords
from django.contrib.auth.models import AbstractUser
from django.db.models import Avg
from .tasks import email_users, notify_order_submitted
from django.conf import settings
from django.core.validators import MinValueValidator, MaxValueValidator
import logging
from django.urls import reverse
from solo.models import SingletonModel
from django.core.exceptions import ValidationError

from imagekit.models import ImageSpecField
from imagekit.processors import ResizeToFill

hashed_storage = HashedFilenameStorage()


class Setting(SingletonModel):
    site_name = models.CharField(max_length=255, default="Sandwitches")
    site_description = models.TextField(blank=True, null=True)
    email = models.EmailField(blank=True, null=True)
    ai_connection_point = models.URLField(blank=True, null=True)
    ai_model = models.CharField(max_length=255, blank=True, null=True)
    ai_api_key = models.CharField(max_length=255, blank=True, null=True)

    def __str__(self):
        return "Site Settings"

    class Meta:
        verbose_name = "Site Settings"


class User(AbstractUser):
    avatar = models.ImageField(upload_to="avatars", blank=True, null=True)
    avatar_thumbnail = ImageSpecField(
        source="avatar",
        processors=[ResizeToFill(100, 50)],
        format="JPEG",
        options={"quality": 60},
    )
    bio = models.TextField(blank=True)
    language = models.CharField(
        max_length=10,
        choices=settings.LANGUAGES,
        default=settings.LANGUAGE_CODE,
    )
    favorites = models.ManyToManyField(
        "Recipe", related_name="favorited_by", blank=True
    )

    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"

    def __str__(self):
        return self.username


class Tag(models.Model):
    name = models.CharField(max_length=50, unique=True)
    slug = models.SlugField(max_length=60, unique=True, blank=True)

    class Meta:
        ordering = ("name",)
        verbose_name = "Tag"
        verbose_name_plural = "Tags"

    def save(self, *args, **kwargs):
        if not self.slug:
            base = slugify(self.name)[:55]
            slug = base
            n = 1
            while Tag.objects.filter(slug=slug).exclude(pk=self.pk).exists():  # ty:ignore[unresolved-attribute]
                slug = f"{base}-{n}"
                n += 1
            self.slug = slug
        super().save(*args, **kwargs)

    def __str__(self):
        return self.name


class Recipe(models.Model):
    title = models.CharField(max_length=255, unique=True)
    slug = models.SlugField(max_length=255, unique=True, blank=True)
    description = models.TextField(blank=True)
    ingredients = models.TextField(blank=True)
    instructions = models.TextField(blank=True)
    servings = models.IntegerField(default=1, validators=[MinValueValidator(1)])
    price = models.DecimalField(
        max_digits=6, decimal_places=2, null=True, blank=True, verbose_name="Price (€)"
    )
    uploaded_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="recipes",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    image = models.ImageField(
        upload_to="recipes/",
        storage=hashed_storage,
        blank=True,
        null=True,
    )
    image_thumbnail = ImageSpecField(
        source="image",
        processors=[ResizeToFill(150, 150)],
        format="JPEG",
        options={"quality": 70},
    )
    image_small = ImageSpecField(
        source="image",
        processors=[ResizeToFill(400, 300)],
        format="JPEG",
        options={"quality": 75},
    )
    image_medium = ImageSpecField(
        source="image",
        processors=[ResizeToFill(700, 500)],
        format="JPEG",
        options={"quality": 85},
    )
    image_large = ImageSpecField(
        source="image",
        processors=[ResizeToFill(1200, 800)],
        format="JPEG",
        options={"quality": 95},
    )
    tags = models.ManyToManyField(Tag, blank=True, related_name="recipes")
    is_highlighted = models.BooleanField(default=False)
    is_approved = models.BooleanField(default=False)
    max_daily_orders = models.PositiveIntegerField(
        null=True, blank=True, verbose_name="Max daily orders"
    )
    daily_orders_count = models.PositiveIntegerField(default=0)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    history = HistoricalRecords()

    class Meta:
        ordering = ("-created_at",)
        verbose_name = "Recipe"
        verbose_name_plural = "Recipes"

    def save(self, *args, **kwargs):
        is_new = self._state.adding

        if not self.slug:
            base = slugify(self.title)[:240]
            slug = base
            n = 1
            while Recipe.objects.filter(slug=slug).exclude(pk=self.pk).exists():  # ty:ignore[unresolved-attribute]
                slug = f"{base}-{n}"
                n += 1
            self.slug = slug

        super().save(*args, **kwargs)

        send_email = getattr(settings, "SEND_EMAIL")
        logging.debug(f"SEND_EMAIL is set to {send_email}")

        if is_new or settings.DEBUG:
            if send_email:
                email_users.enqueue(recipe_id=self.pk)
            else:
                logging.warning(
                    "Email sending is disabled; not sending email notification, make sure SEND_EMAIL is set to True in settings."
                )
        else:
            logging.debug(
                "Existing recipe saved (update); skipping email notification."
            )

    def get_absolute_url(self):
        return reverse("recipe_detail", kwargs={"slug": self.slug})

    def tag_list(self):
        return list(self.tags.values_list("name", flat=True))  # ty:ignore[possibly-missing-attribute]

    def set_tags_from_string(self, tag_string):
        """
        Accepts a comma separated string like "tag1, tag2" and attaches existing tags
        or creates new ones as needed. Returns the Tag queryset assigned.
        """
        names = [t.strip() for t in (tag_string or "").split(",") if t.strip()]
        tags = []
        for name in names:
            tag = Tag.objects.filter(name__iexact=name).first()  # ty:ignore[unresolved-attribute]
            if not tag:
                tag = Tag.objects.create(name=name)  # ty:ignore[unresolved-attribute]
            tags.append(tag)
        self.tags.set(tags)  # ty:ignore[possibly-missing-attribute]
        return self.tags.all()  # ty:ignore[possibly-missing-attribute]

    def average_rating(self):
        agg = self.ratings.aggregate(avg=Avg("score"))  # ty:ignore[unresolved-attribute]
        return agg["avg"] or 0

    def rating_count(self):
        return self.ratings.count()  # ty:ignore[unresolved-attribute]

    def __str__(self):
        return self.title


class Rating(models.Model):
    recipe = models.ForeignKey(Recipe, related_name="ratings", on_delete=models.CASCADE)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name="ratings", on_delete=models.CASCADE
    )
    score = models.FloatField(
        validators=[MinValueValidator(0.0), MaxValueValidator(10.0)]
    )
    comment = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = ("recipe", "user")
        ordering = ("-updated_at",)

    def __str__(self):
        return f"{self.recipe} — {self.score} by {self.user}"


class Order(models.Model):
    STATUS_CHOICES = (
        ("PENDING", "Pending"),
        ("COMPLETED", "Completed"),
        ("CANCELLED", "Cancelled"),
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name="orders", on_delete=models.CASCADE
    )
    recipe = models.ForeignKey(Recipe, related_name="orders", on_delete=models.CASCADE)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default="PENDING")
    total_price = models.DecimalField(max_digits=6, decimal_places=2)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        ordering = ("-created_at",)
        verbose_name = "Order"
        verbose_name_plural = "Orders"

    def save(self, *args, **kwargs):
        if not self.recipe.price:  # ty:ignore[possibly-missing-attribute]
            raise ValueError("Cannot order a recipe without a price.")
        if not self.total_price:
            self.total_price = self.recipe.price  # ty:ignore[possibly-missing-attribute]

        is_new = self.pk is None
        if is_new:
            # We use select_for_update to lock the row and prevent race conditions
            # However, since 'self.recipe' is already fetched, we need to re-fetch it with lock if we want to be strict.
            # For simplicity in this context, we will reload it or trust the current instance but ideally:

            # We need to wrap this in a transaction if not already
            # But simple increment logic:
            if (
                self.recipe.max_daily_orders is not None  # ty:ignore[possibly-missing-attribute]
                and self.recipe.daily_orders_count >= self.recipe.max_daily_orders  # ty:ignore[possibly-missing-attribute]
            ):
                raise ValidationError("Daily order limit reached for this recipe.")

            self.recipe.daily_orders_count += 1  # ty:ignore[possibly-missing-attribute]
            self.recipe.save(update_fields=["daily_orders_count"])  # ty:ignore[possibly-missing-attribute]

        super().save(*args, **kwargs)

        if is_new:
            notify_order_submitted.enqueue(order_id=self.pk)

    def __str__(self):
        return f"Order #{self.pk} - {self.user} - {self.recipe}"


class CartItem(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name="cart_items", on_delete=models.CASCADE
    )
    recipe = models.ForeignKey(
        Recipe, related_name="cart_items", on_delete=models.CASCADE
    )
    quantity = models.PositiveIntegerField(default=1)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = ("user", "recipe")
        verbose_name = "Cart Item"
        verbose_name_plural = "Cart Items"

    def __str__(self):
        return f"{self.user.username}'s cart: {self.recipe.title} (x{self.quantity})"

    @property
    def total_price(self):
        if self.recipe.price:
            return self.recipe.price * self.quantity
        return 0



import markdown

from django import template
from django.template.defaultfilters import stringfilter

register = template.Library()


@register.filter
@stringfilter
def convert_markdown(value):
    md = markdown.markdown(
        value,
        extensions=["markdown.extensions.fenced_code", "markdown.extensions.tables"],
    )

    return md



import logging
from ninja import NinjaAPI
from .models import Recipe, Tag, Setting, Rating, Order
from django.contrib.auth import get_user_model
from .utils import (
    parse_ingredient_line,
    scale_ingredient,
    format_scaled_ingredient,
)  # Import utility functions

from ninja import ModelSchema
from ninja import Schema
from django.shortcuts import get_object_or_404
from datetime import date
import random
from typing import List, Optional  # Import typing hints
from django.core.exceptions import ValidationError

from ninja.security import django_auth

from . import __version__

# Get the custom User model
User = get_user_model()

api = NinjaAPI(version=__version__)


class UserPublicSchema(ModelSchema):
    class Meta:
        model = User
        fields = ["username", "first_name", "last_name", "avatar"]


class RecipeSchema(ModelSchema):
    favorited_by: List[UserPublicSchema] = []

    class Meta:
        model = Recipe
        fields = "__all__"


class TagSchema(ModelSchema):
    class Meta:
        model = Tag
        fields = "__all__"


class UserSchema(ModelSchema):
    class Meta:
        model = User
        exclude = ["password", "last_login", "user_permissions"]


class SettingSchema(ModelSchema):
    class Meta:
        model = Setting
        fields = "__all__"


class RatingSchema(ModelSchema):
    user: UserPublicSchema

    class Meta:
        model = Rating
        fields = "__all__"


class Error(Schema):
    message: str


class RatingResponseSchema(Schema):
    average: float
    count: int


class ScaledIngredient(Schema):  # New Schema for scaled ingredients
    original_line: str
    scaled_line: str
    quantity: Optional[float]
    unit: Optional[str]
    name: Optional[str]


class OrderSchema(ModelSchema):
    class Meta:
        model = Order
        fields = ["id", "status", "total_price", "created_at"]


class CreateOrderSchema(Schema):
    recipe_id: int


@api.get("ping")
def ping(request):
    return {"status": "ok", "message": "pong"}


@api.get("v1/settings", response=SettingSchema)
def get_settings(request):
    return Setting.objects.get()  # ty:ignore[unresolved-attribute]


@api.post("v1/settings", auth=django_auth, response={200: SettingSchema, 403: Error})
def update_settings(request, payload: SettingSchema):
    if not request.user.is_staff:
        return 403, {"message": "You are not authorized to perform this action"}
    settings = Setting.objects.get()  # ty:ignore[unresolved-attribute]
    for attr, value in payload.dict().items():
        setattr(settings, attr, value)
    settings.save()
    return settings


@api.get("v1/me", response={200: UserSchema, 403: Error})
def me(request):
    if not request.user.is_authenticated:
        return 403, {"message": "Please sign in first"}
    return request.user


@api.get("v1/users", auth=django_auth, response=list[UserSchema])
def users(request):
    return User.objects.all()


@api.get("v1/recipes", response=list[RecipeSchema])
def get_recipes(request):
    return Recipe.objects.all().prefetch_related("favorited_by")  # ty:ignore[unresolved-attribute]


@api.get("v1/recipes/{recipe_id}", response=RecipeSchema)
def get_recipe(request, recipe_id: int):
    recipe = get_object_or_404(
        Recipe.objects.prefetch_related("favorited_by"),  # ty:ignore[unresolved-attribute]
        id=recipe_id,
    )
    return recipe


@api.get("v1/recipes/{recipe_id}/scale-ingredients", response=List[ScaledIngredient])
def scale_recipe_ingredients(request, recipe_id: int, target_servings: int):
    recipe = get_object_or_404(Recipe, id=recipe_id)

    # Ensure target_servings is at least 1
    target_servings = max(1, target_servings)

    current_servings = recipe.servings
    if not current_servings or current_servings <= 0:
        current_servings = 1

    ingredient_lines = [
        line.strip() for line in (recipe.ingredients or "").split("\n") if line.strip()
    ]

    scaled_ingredients_output = []
    for line in ingredient_lines:
        try:
            parsed = parse_ingredient_line(line)
            scaled = scale_ingredient(parsed, current_servings, target_servings)
            formatted_line = format_scaled_ingredient(scaled)

            scaled_ingredients_output.append(
                ScaledIngredient(
                    original_line=line,
                    scaled_line=formatted_line,
                    quantity=scaled.get("quantity"),
                    unit=scaled.get("unit"),
                    name=scaled.get("name"),
                )
            )
        except Exception as e:
            # Fallback for lines that fail to parse/scale
            scaled_ingredients_output.append(
                ScaledIngredient(
                    original_line=line,
                    scaled_line=line,
                    quantity=None,
                    unit=None,
                    name=line,
                )
            )
            logging.warning(f"Failed to scale ingredient line '{line}': {e}")

    return scaled_ingredients_output


@api.get("v1/recipe-of-the-day", response=RecipeSchema)
def get_recipe_of_the_day(request):
    recipes = list(Recipe.objects.all().prefetch_related("favorited_by"))  # ty:ignore[unresolved-attribute]
    if not recipes:
        return None
    today = date.today()
    random.seed(today.toordinal())
    recipe = random.choice(recipes)
    return recipe


@api.get("v1/recipes/{recipe_id}/rating", response=RatingResponseSchema)
def get_recipe_rating(request, recipe_id: int):
    recipe = get_object_or_404(Recipe, id=recipe_id)
    return {
        "average": recipe.average_rating(),
        "count": recipe.rating_count(),
    }


@api.get("v1/tags", response=list[TagSchema])
def get_tags(request):
    return Tag.objects.all()  # ty:ignore[unresolved-attribute]


@api.get("v1/tags/{tag_id}", response=TagSchema)
def get_tag(request, tag_id: int):
    tag = get_object_or_404(Tag, id=tag_id)
    return tag


@api.post("v1/orders", auth=django_auth, response={201: OrderSchema, 400: Error})
def create_order(request, payload: CreateOrderSchema):
    recipe = get_object_or_404(Recipe, id=payload.recipe_id)
    try:
        order = Order.objects.create(user=request.user, recipe=recipe)  # ty:ignore[unresolved-attribute]
        return 201, order
    except (ValidationError, ValueError) as e:
        return 400, {"message": str(e)}



# Generated by Django 6.0.1 on 2026-01-24 16:01

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "sandwitches",
            "0010_rename_is_approved_historicalrecipe_is_community_made_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalrecipe",
            name="is_community_made",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="recipe",
            name="is_community_made",
            field=models.BooleanField(default=False),
        ),
    ]



import re

# Define a set of common units for better parsing
COMMON_UNITS = {
    "cup",
    "cups",
    "oz",
    "ounce",
    "ounces",
    "g",
    "gram",
    "grams",
    "kg",
    "kilogram",
    "kilograms",
    "lb",
    "lbs",
    "pound",
    "pounds",
    "ml",
    "milliliter",
    "milliliters",
    "l",
    "liter",
    "liters",
    "tsp",
    "teaspoon",
    "teaspoons",
    "tbsp",
    "tablespoon",
    "tablespoons",
    "pinch",
    "pinches",
    "slice",
    "slices",
    "clove",
    "cloves",
    "large",
    "medium",
    "small",
    "can",
    "cans",
    "package",
    "packages",
    "piece",
    "pieces",
    "dash",
    "dashes",
    "sprig",
    "sprigs",
    "to taste",
}


def parse_ingredient_line(line):
    """
    Parses a single ingredient line to extract quantity, unit, and ingredient name.
    This is a heuristic approach and may not cover all possible formats.
    Returns a dictionary with 'quantity', 'unit', 'name', and 'original_line'.
    Quantity will be a float, unit and name strings.
    If parsing fails, returns None for quantity/unit but includes original_line.
    """
    original_line = line.strip()

    quantity = None
    unit = None
    name = original_line

    # Regex to capture quantity at the beginning:
    # 1. Whole number with fraction (e.g., "1 1/2", "2 3/4")
    # 2. Simple fraction (e.g., "1/2", "3/4")
    # 3. Decimal number (e.g., "0.5", "2.0", ".75")
    # 4. Whole number (e.g., "1", "100")
    # The quantity is optional to handle lines like "Salt to taste".
    #
    # This pattern specifically tries to get the quantity part.
    # It covers:
    # - `1 1/2` (whole number followed by space and fraction)
    # - `1/2` (fraction)
    # - `1.5` (decimal)
    # - `1` (integer)
    quantity_match = re.match(
        r"^\s*(\d+\s+\d/\d|\d/\d|\d+\.?\d*|\.\d+)\s*(.*)", original_line
    )

    remaining_line = original_line

    if quantity_match:
        quantity_str = quantity_match.group(1)
        remaining_line = quantity_match.group(2).strip()

        try:
            # Handle fractions like "1 1/2" or "1/2"
            if " " in quantity_str and "/" in quantity_str:
                parts = quantity_str.split(" ", 1)
                whole = float(parts[0])
                fraction_parts = parts[1].split("/")
                numerator = float(fraction_parts[0])
                denominator = float(fraction_parts[1])
                quantity = whole + (numerator / denominator)
            elif "/" in quantity_str:
                parts = quantity_str.split("/")
                numerator = float(parts[0])
                denominator = float(parts[1])
                quantity = numerator / denominator
            else:
                quantity = float(quantity_str)
        except ValueError:
            quantity = None  # Parsing quantity failed

    # Now, try to find a unit in the remaining_line
    # We iterate through common units to find the longest match first
    found_unit = None
    for common_unit in sorted(list(COMMON_UNITS), key=len, reverse=True):
        # Check if the unit is at the beginning of the remaining_line, followed by a space or end of string
        # using word boundary to avoid partial matches (e.g., "cup" in "cupholder")
        unit_regex = r"^\s*" + re.escape(common_unit) + r"(\b|\s|$)"  # ty:ignore[invalid-argument-type]
        unit_match = re.match(unit_regex, remaining_line, re.IGNORECASE)
        if unit_match:
            found_unit = remaining_line[: unit_match.end(1)].strip()
            remaining_line = remaining_line[unit_match.end(1) :].strip()
            break  # Found the unit, stop searching

    if found_unit:
        unit = found_unit

    # The rest of the line is the name
    name = (
        remaining_line
        if remaining_line
        else original_line
        if quantity is None and unit is None
        else ""
    )

    # If quantity was not parsed, and no unit was found, then the whole line is the name
    if quantity is None and unit is None:
        name = original_line
    elif name == "" and (quantity is not None or unit is not None):
        # If we parsed a quantity or unit but no name, it means the name was part of the unit search
        # or it was implicitly part of the quantity_match that took everything.
        # This is a fallback to ensure something is in name if it's not a unit.
        if unit and original_line.lower().startswith(
            f"{quantity_str.lower() if quantity_str else ''} {unit.lower()}".strip()
        ):
            name_start_index = original_line.lower().find(unit.lower()) + len(unit)
            potential_name = original_line[name_start_index:].strip()
            if potential_name:
                name = potential_name
        elif quantity_match:
            # Fallback if name is empty and quantity was matched, it means quantity was everything
            # or the unit was consumed.
            if not unit:  # If no unit, the remaining_line should be the name
                name = remaining_line

    return {
        "quantity": quantity,
        "unit": unit,
        "name": name.strip(),
        "original_line": original_line,
    }


def scale_ingredient(parsed_ingredient, current_servings, target_servings):
    """
    Scales a parsed ingredient based on current and target servings.
    """
    if parsed_ingredient["quantity"] is None or current_servings == 0:
        # Cannot scale if quantity is unknown or current servings is zero
        return parsed_ingredient

    scale_factor = target_servings / current_servings
    scaled_quantity = parsed_ingredient["quantity"] * scale_factor

    return {
        "quantity": scaled_quantity,
        "unit": parsed_ingredient["unit"],
        "name": parsed_ingredient["name"],
        "original_line": parsed_ingredient[
            "original_line"
        ],  # Keep original line reference
    }


def format_scaled_ingredient(scaled_ingredient):
    """
    Formats a scaled ingredient back into a human-readable string.
    This will try to reconstruct the string based on the parsed components.
    """
    quantity = scaled_ingredient["quantity"]
    unit = scaled_ingredient["unit"]
    name = scaled_ingredient["name"]

    if quantity is None:
        return scaled_ingredient[
            "original_line"
        ]  # Fallback to original if quantity was not parsed

    # Simple formatting for now. Could add fraction handling later.
    if quantity == int(quantity):
        quantity_str = str(int(quantity))
    else:
        # Format to 2 decimal places, remove trailing zeros, and remove trailing dot if integer
        quantity_str = f"{quantity:.2f}".rstrip("0").rstrip(".")
        if not quantity_str:  # Handle cases like 0.0 -> ""
            quantity_str = "0"

    parts = []
    if quantity_str:
        parts.append(quantity_str)

    # Check for units that usually don't have a space before them
    if unit:
        if unit.lower() in ["g", "mg"]:  # Only g and mg are typically appended directly
            parts[-1] += unit  # Append unit directly to the last part (quantity)
        else:
            parts.append(unit)

    if name:
        parts.append(name)

    return " ".join(parts).strip()



# Generated by Django 6.0 on 2025-12-31 16:09

import django.contrib.auth.models
import django.contrib.auth.validators
import django.core.validators
import django.db.models.deletion
import django.utils.timezone
import sandwitches.storage
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50, unique=True)),
                ("slug", models.SlugField(blank=True, max_length=60, unique=True)),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
                "ordering": ("name",),
            },
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "username",
                    models.CharField(
                        error_messages={
                            "unique": "A user with that username already exists."
                        },
                        help_text="Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.",
                        max_length=150,
                        unique=True,
                        validators=[
                            django.contrib.auth.validators.UnicodeUsernameValidator()
                        ],
                        verbose_name="username",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=150, verbose_name="first name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=150, verbose_name="last name"
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="email address"
                    ),
                ),
                (
                    "is_staff",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether the user can log into this admin site.",
                        verbose_name="staff status",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Designates whether this user should be treated as active. Unselect this instead of deleting accounts.",
                        verbose_name="active",
                    ),
                ),
                (
                    "date_joined",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="date joined"
                    ),
                ),
                (
                    "avatar",
                    models.ImageField(blank=True, null=True, upload_to="avatars"),
                ),
                ("bio", models.TextField(blank=True)),
                (
                    "language",
                    models.CharField(
                        choices=[("en", "English"), ("nl", "Nederlands")],
                        default="en",
                        max_length=10,
                    ),
                ),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "verbose_name": "User",
                "verbose_name_plural": "Users",
            },
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalRecipe",
            fields=[
                (
                    "id",
                    models.BigIntegerField(
                        auto_created=True, blank=True, db_index=True, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(db_index=True, max_length=255)),
                ("slug", models.SlugField(blank=True, max_length=255)),
                ("description", models.TextField(blank=True)),
                ("ingredients", models.TextField(blank=True)),
                ("instructions", models.TextField(blank=True)),
                ("image", models.TextField(blank=True, max_length=100, null=True)),
                ("created_at", models.DateTimeField(blank=True, editable=False)),
                ("updated_at", models.DateTimeField(blank=True, editable=False)),
                ("history_id", models.AutoField(primary_key=True, serialize=False)),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "uploaded_by",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Recipe",
                "verbose_name_plural": "historical Recipes",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="Recipe",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255, unique=True)),
                ("slug", models.SlugField(blank=True, max_length=255, unique=True)),
                ("description", models.TextField(blank=True)),
                ("ingredients", models.TextField(blank=True)),
                ("instructions", models.TextField(blank=True)),
                (
                    "image",
                    models.ImageField(
                        blank=True,
                        null=True,
                        storage=sandwitches.storage.HashedFilenameStorage(),
                        upload_to="recipes/",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "uploaded_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="recipes",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "tags",
                    models.ManyToManyField(
                        blank=True, related_name="recipes", to="sandwitches.tag"
                    ),
                ),
            ],
            options={
                "verbose_name": "Recipe",
                "verbose_name_plural": "Recipes",
                "ordering": ("-created_at",),
            },
        ),
        migrations.AddField(
            model_name="user",
            name="favorites",
            field=models.ManyToManyField(
                blank=True, related_name="favorited_by", to="sandwitches.recipe"
            ),
        ),
        migrations.CreateModel(
            name="Rating",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "score",
                    models.FloatField(
                        validators=[
                            django.core.validators.MinValueValidator(0.0),
                            django.core.validators.MaxValueValidator(10.0),
                        ]
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ratings",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "recipe",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ratings",
                        to="sandwitches.recipe",
                    ),
                ),
            ],
            options={
                "ordering": ("-updated_at",),
                "unique_together": {("recipe", "user")},
            },
        ),
    ]



import logging
from django.core.exceptions import ValidationError
from django.shortcuts import render, get_object_or_404, redirect
from django.urls import reverse
from django.contrib import messages
from django.contrib.auth import login
from django.contrib.auth import get_user_model
from django.contrib.auth.decorators import login_required
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.translation import gettext as _
from .models import Recipe, Rating, Tag, Order, CartItem
from .forms import (
    RecipeForm,
    AdminSetupForm,
    UserSignupForm,
    RatingForm,
    UserEditForm,
    TagForm,
    UserProfileForm,
    UserRecipeSubmissionForm,
)
from django.http import HttpResponseBadRequest, Http404
from django.conf import settings
from django.http import FileResponse
from pathlib import Path
import mimetypes
from PIL import Image
from django.db.models import Q, Avg
from django_tasks.backends.database.models import DBTaskResult
from django.contrib.auth.views import LoginView


from sandwitches import __version__ as sandwitches_version

User = get_user_model()


@login_required
def community(request):
    if request.method == "POST":
        form = UserRecipeSubmissionForm(request.POST, request.FILES)
        if form.is_valid():
            recipe = form.save(commit=False)
            recipe.uploaded_by = request.user
            recipe.is_approved = False
            recipe.save()
            form.save_m2m()
            messages.success(
                request,
                _("Your recipe has been submitted and is awaiting admin approval."),
            )
            return redirect("user_profile")
    else:
        form = UserRecipeSubmissionForm()

    # Community recipes = uploaded by users in 'community' group
    recipes = Recipe.objects.filter(  # ty:ignore[unresolved-attribute]
        uploaded_by__groups__name="community"
    ).prefetch_related("favorited_by")

    if not (request.user.is_staff or request.user.groups.filter(name="admin").exists()):
        # Regular users only see approved community recipes or their own
        recipes = recipes.filter(Q(is_approved=True) | Q(uploaded_by=request.user))
    else:
        # Admins see all community recipes
        pass

    recipes = recipes.order_by("-created_at")

    return render(
        request,
        "community.html",
        {
            "form": form,
            "recipes": recipes,
            "title": _("Community"),
            "version": sandwitches_version,
            "user": request.user,
        },
    )


class CustomLoginView(LoginView):
    template_name = "login.html"
    redirect_authenticated_user = True


@staff_member_required
def admin_dashboard(request):
    recipe_count = Recipe.objects.count()  # ty:ignore[unresolved-attribute]
    user_count = User.objects.count()
    tag_count = Tag.objects.count()  # ty:ignore[unresolved-attribute]
    recent_recipes = Recipe.objects.order_by("-created_at")[:5]  # ty:ignore[unresolved-attribute]

    # Data for charts
    from datetime import timedelta
    from django.utils import timezone
    from django.db.models.functions import TruncDate
    from django.db.models import Count

    # Get date range from request or default to last 30 days
    end_date_str = request.GET.get("end_date")
    start_date_str = request.GET.get("start_date")

    today = timezone.now().date()
    try:
        end_date = (
            timezone.datetime.strptime(end_date_str, "%Y-%m-%d").date()
            if end_date_str
            else today
        )
        start_date = (
            timezone.datetime.strptime(start_date_str, "%Y-%m-%d").date()
            if start_date_str
            else end_date - timedelta(days=30)
        )
    except (ValueError, TypeError):
        end_date = today
        start_date = today - timedelta(days=30)

    # Recipes over time
    recipe_data = (
        Recipe.objects.filter(created_at__date__range=(start_date, end_date))  # ty:ignore[unresolved-attribute]
        .annotate(date=TruncDate("created_at"))
        .values("date")
        .annotate(count=Count("id"))
        .order_by("date")
    )

    # Ratings over time
    rating_data = (
        Rating.objects.filter(created_at__date__range=(start_date, end_date))  # ty:ignore[unresolved-attribute]
        .annotate(date=TruncDate("created_at"))
        .values("date")
        .annotate(avg=Avg("score"))
        .order_by("date")
    )

    # Orders over time
    order_data = (
        Order.objects.filter(created_at__date__range=(start_date, end_date))  # ty:ignore[unresolved-attribute]
        .annotate(date=TruncDate("created_at"))
        .values("date")
        .annotate(count=Count("id"))
        .order_by("date")
    )

    # Prepare labels and data for JS
    recipe_labels = [d["date"].strftime("%d/%m/%Y") for d in recipe_data]
    recipe_counts = [d["count"] for d in recipe_data]

    rating_labels = [d["date"].strftime("%d/%m/%Y") for d in rating_data]
    rating_avgs = [float(d["avg"]) for d in rating_data]

    order_labels = [d["date"].strftime("%d/%m/%Y") for d in order_data]
    order_counts = [d["count"] for d in order_data]

    pending_recipes = Recipe.objects.filter(  # ty:ignore[unresolved-attribute]
        is_approved=False, uploaded_by__groups__name="community"
    ).order_by("-created_at")
    context = {
        "recipe_count": recipe_count,
        "user_count": user_count,
        "tag_count": tag_count,
        "recent_recipes": recent_recipes,
        "pending_recipes": pending_recipes,
        "recipe_labels": recipe_labels,
        "recipe_counts": recipe_counts,
        "rating_labels": rating_labels,
        "rating_avgs": rating_avgs,
        "order_labels": order_labels,
        "order_counts": order_counts,
        "start_date": start_date.strftime("%Y-%m-%d"),
        "end_date": end_date.strftime("%Y-%m-%d"),
        "version": sandwitches_version,
    }

    if request.headers.get("HX-Request"):
        return render(request, "admin/partials/dashboard_charts.html", context)

    return render(
        request,
        "admin/dashboard.html",
        context,
    )


@staff_member_required
def admin_recipe_list(request):
    sort_param = request.GET.get("sort", "-created_at")
    allowed_sorts = {
        "title": "title",
        "-title": "-title",
        "created_at": "created_at",
        "-created_at": "-created_at",
        "uploader": "uploaded_by__username",
        "-uploader": "-uploaded_by__username",
        "price": "price",
        "-price": "-price",
        "orders": "daily_orders_count",
        "-orders": "-daily_orders_count",
        "rating": "avg_rating",
        "-rating": "-avg_rating",
    }

    order_by = allowed_sorts.get(sort_param, "-created_at")

    recipes = (
        Recipe.objects.annotate(avg_rating=Avg("ratings__score"))  # ty:ignore[unresolved-attribute]
        .prefetch_related("tags")
        .order_by(order_by)
    )
    return render(
        request,
        "admin/recipe_list.html",
        {
            "recipes": recipes,
            "version": sandwitches_version,
            "current_sort": sort_param,
        },
    )


@staff_member_required
def admin_recipe_approval_list(request):
    recipes = Recipe.objects.filter(  # ty:ignore[unresolved-attribute]
        is_approved=False, uploaded_by__groups__name="community"
    ).order_by("-created_at")
    return render(
        request,
        "admin/recipe_approval_list.html",
        {
            "recipes": recipes,
            "version": sandwitches_version,
        },
    )


@staff_member_required
def admin_recipe_add(request):
    if request.method == "POST":
        form = RecipeForm(request.POST, request.FILES)
        if form.is_valid():
            recipe = form.save(commit=False)
            recipe.uploaded_by = request.user
            recipe.save()
            form.save_m2m()
            messages.success(request, _("Recipe added successfully."))
            return redirect("admin_recipe_list")
    else:
        form = RecipeForm()
    return render(
        request,
        "admin/recipe_form.html",
        {"form": form, "title": _("Add Recipe"), "version": sandwitches_version},
    )


@staff_member_required
def admin_recipe_edit(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    if request.method == "POST":
        form = RecipeForm(request.POST, request.FILES, instance=recipe)
        if form.is_valid():
            form.save()
            messages.success(request, _("Recipe updated successfully."))
            return redirect("admin_recipe_list")
    else:
        form = RecipeForm(instance=recipe)
    return render(
        request,
        "admin/recipe_form.html",
        {
            "form": form,
            "recipe": recipe,
            "title": _("Edit Recipe"),
            "version": sandwitches_version,
        },
    )


@staff_member_required
def admin_recipe_approve(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    recipe.is_approved = True
    recipe.save()
    messages.success(
        request, _("Recipe '%(title)s' approved.") % {"title": recipe.title}
    )
    referer = request.META.get("HTTP_REFERER")
    if referer and "dashboard/approvals" in referer:
        return redirect("admin_recipe_approval_list")
    return redirect("admin_recipe_list")


@staff_member_required
def admin_recipe_delete(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    if request.method == "POST":
        recipe.delete()
        messages.success(request, _("Recipe deleted."))
        return redirect("admin_recipe_list")
    return render(
        request,
        "admin/confirm_delete.html",
        {"object": recipe, "type": _("recipe"), "version": sandwitches_version},
    )


@staff_member_required
def admin_recipe_rotate(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    direction = request.GET.get("direction", "cw")
    if not recipe.image:
        messages.error(request, _("No image to rotate."))
        return redirect("admin_recipe_edit", pk=pk)

    try:
        img = Image.open(recipe.image.path)
        if direction == "ccw":
            # Rotate 90 degrees counter-clockwise
            img = img.rotate(90, expand=True)
        else:
            # Rotate 90 degrees clockwise (default)
            img = img.rotate(-90, expand=True)
        img.save(recipe.image.path)
        messages.success(request, _("Image rotated successfully."))
    except Exception as e:
        messages.error(request, _("Error rotating image: ") + str(e))

    return redirect("admin_recipe_edit", pk=pk)


@staff_member_required
def admin_user_list(request):
    users = User.objects.all()
    return render(
        request,
        "admin/user_list.html",
        {"users": users, "version": sandwitches_version},
    )


@staff_member_required
def admin_user_edit(request, pk):
    user_obj = get_object_or_404(User, pk=pk)
    if request.method == "POST":
        form = UserEditForm(request.POST, request.FILES, instance=user_obj)
        if form.is_valid():
            form.save()
            messages.success(request, _("User updated successfully."))
            return redirect("admin_user_list")
    else:
        form = UserEditForm(instance=user_obj)
    return render(
        request,
        "admin/user_form.html",
        {"form": form, "user_obj": user_obj, "version": sandwitches_version},
    )


@staff_member_required
def admin_user_delete(request, pk):
    user_obj = get_object_or_404(User, pk=pk)
    if user_obj == request.user:
        messages.error(request, _("You cannot delete yourself."))
        return redirect("admin_user_list")
    if request.method == "POST":
        user_obj.delete()
        messages.success(request, _("User deleted."))
        return redirect("admin_user_list")
    return render(
        request,
        "admin/confirm_delete.html",
        {"object": user_obj, "type": _("user"), "version": sandwitches_version},
    )


@staff_member_required
def admin_tag_list(request):
    tags = Tag.objects.all()  # ty:ignore[unresolved-attribute]
    return render(
        request,
        "admin/tag_list.html",
        {"tags": tags, "version": sandwitches_version},
    )


@staff_member_required
def admin_tag_add(request):
    if request.method == "POST":
        form = TagForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, _("Tag added successfully."))
            return redirect("admin_tag_list")
    else:
        form = TagForm()
    return render(
        request,
        "admin/tag_form.html",
        {"form": form, "title": _("Add Tag"), "version": sandwitches_version},
    )


@staff_member_required
def admin_tag_edit(request, pk):
    tag = get_object_or_404(Tag, pk=pk)
    if request.method == "POST":
        form = TagForm(request.POST, instance=tag)
        if form.is_valid():
            form.save()
            messages.success(request, _("Tag updated successfully."))
            return redirect("admin_tag_list")
    else:
        form = TagForm(instance=tag)
    return render(
        request,
        "admin/tag_form.html",
        {
            "form": form,
            "tag": tag,
            "title": _("Edit Tag"),
            "version": sandwitches_version,
        },
    )


@staff_member_required
def admin_tag_delete(request, pk):
    tag = get_object_or_404(Tag, pk=pk)
    if request.method == "POST":
        tag.delete()
        messages.success(request, _("Tag deleted."))
        return redirect("admin_tag_list")
    return render(
        request,
        "admin/confirm_delete.html",
        {"object": tag, "type": _("tag"), "version": sandwitches_version},
    )


@staff_member_required
def admin_task_list(request):
    tasks = DBTaskResult.objects.all().order_by("-enqueued_at")[:50]
    return render(
        request,
        "admin/task_list.html",
        {"tasks": tasks, "version": sandwitches_version},
    )


@staff_member_required
def admin_task_detail(request, pk):
    task = get_object_or_404(DBTaskResult, pk=pk)
    return render(
        request,
        "admin/task_detail.html",
        {"task": task, "version": sandwitches_version},
    )


@staff_member_required
def admin_rating_list(request):
    ratings = (
        Rating.objects.select_related("recipe", "user")  # ty:ignore[unresolved-attribute]
        .all()
        .order_by("-updated_at")
    )
    return render(
        request,
        "admin/rating_list.html",
        {"ratings": ratings, "version": sandwitches_version},
    )


@staff_member_required
def admin_rating_delete(request, pk):
    rating = get_object_or_404(Rating, pk=pk)
    if request.method == "POST":
        rating.delete()
        messages.success(request, _("Rating deleted."))
        return redirect("admin_rating_list")
    return render(
        request,
        "admin/confirm_delete.html",
        {"object": rating, "type": _("rating"), "version": sandwitches_version},
    )


@staff_member_required
def admin_order_list(request):
    orders = (
        Order.objects.select_related("user", "recipe")  # ty:ignore[unresolved-attribute]
        .all()
        .order_by("-created_at")
    )

    if request.headers.get("HX-Request"):
        return render(
            request,
            "admin/partials/order_rows.html",
            {"orders": orders, "version": sandwitches_version},
        )

    return render(
        request,
        "admin/order_list.html",
        {"orders": orders, "version": sandwitches_version},
    )


def recipe_detail(request, slug):
    recipe = get_object_or_404(Recipe, slug=slug)

    # If it's a community recipe, it must be approved or viewed by staff/owner
    is_community = (
        recipe.uploaded_by
        and recipe.uploaded_by.groups.filter(name="community").exists()
    )
    if is_community and not recipe.is_approved:
        if not (
            request.user.is_authenticated
            and (
                request.user.is_staff
                or recipe.uploaded_by == request.user
                or request.user.groups.filter(name="admin").exists()
            )
        ):
            raise Http404("Recipe not found or pending approval.")

    avg = recipe.average_rating()
    count = recipe.rating_count()
    user_rating = None
    rating_form = None
    if request.user.is_authenticated:
        try:
            user_rating = Rating.objects.get(recipe=recipe, user=request.user)  # ty:ignore[unresolved-attribute]
        except Rating.DoesNotExist:  # ty:ignore[unresolved-attribute]
            user_rating = None
        # show form prefilled when possible
        initial = (
            {"score": str(user_rating.score), "comment": user_rating.comment}
            if user_rating
            else None
        )
        rating_form = RatingForm(initial=initial)
    return render(
        request,
        "detail.html",
        {
            "recipe": recipe,
            "avg_rating": avg,
            "rating_count": count,
            "user_rating": user_rating,
            "rating_form": rating_form,
            "version": sandwitches_version,
            "all_ratings": recipe.ratings.select_related("user").order_by(
                "-created_at"
            ),  # Add all ratings for display
        },
    )


@login_required
def order_recipe(request, pk):
    """
    Create an order for the given recipe by the logged-in user.
    """
    recipe = get_object_or_404(Recipe, pk=pk)
    if request.method != "POST":
        return redirect("recipe_detail", slug=recipe.slug)

    try:
        order = Order.objects.create(user=request.user, recipe=recipe)  # ty:ignore[unresolved-attribute]
        logging.debug(f"Created {order}")
        messages.success(
            request,
            _("Your order for %(title)s has been submitted!") % {"title": recipe.title},
        )
    except (ValidationError, ValueError) as e:
        messages.error(request, str(e))

    return redirect("recipe_detail", slug=recipe.slug)


@login_required
def recipe_rate(request, pk):
    """
    Create or update a rating for the given recipe by the logged-in user.
    """
    recipe = get_object_or_404(Recipe, pk=pk)
    if request.method != "POST":
        return redirect("recipe_detail", slug=recipe.slug)

    form = RatingForm(request.POST)
    if form.is_valid():
        score = form.cleaned_data["score"]
        comment = form.cleaned_data["comment"]
        Rating.objects.update_or_create(  # ty:ignore[unresolved-attribute]
            recipe=recipe,
            user=request.user,
            defaults={"score": score, "comment": comment},
        )
        messages.success(request, _("Your rating has been saved."))
    else:
        messages.error(request, _("Could not save rating."))
    return redirect("recipe_detail", slug=recipe.slug)


@login_required
def toggle_favorite(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    if recipe in request.user.favorites.all():
        request.user.favorites.remove(recipe)
        messages.success(request, _("Recipe removed from favorites."))
    else:
        request.user.favorites.add(recipe)
        messages.success(request, _("Recipe added to favorites."))

    # Redirect to the page where the request came from, or default to recipe detail
    referer = request.META.get("HTTP_REFERER")
    if referer:
        return redirect(referer)
    return redirect("recipe_detail", slug=recipe.slug)


@login_required
def favorites(request):
    recipes = request.user.favorites.all().prefetch_related("favorited_by")

    # Filtering
    q = request.GET.get("q")
    if q:
        recipes = recipes.filter(
            Q(title__icontains=q) | Q(tags__name__icontains=q)
        ).distinct()

    date_start = request.GET.get("date_start")
    if date_start:
        recipes = recipes.filter(created_at__gte=date_start)

    date_end = request.GET.get("date_end")
    if date_end:
        recipes = recipes.filter(created_at__lte=date_end)

    uploader = request.GET.get("uploader")
    if uploader:
        recipes = recipes.filter(uploaded_by__username=uploader)

    tag = request.GET.get("tag")
    if tag:
        recipes = recipes.filter(tags__name=tag)

    # Sorting
    sort = request.GET.get("sort", "date_desc")
    if sort == "date_asc":
        recipes = recipes.order_by("created_at")
    elif sort == "rating":
        recipes = recipes.annotate(avg_rating=Avg("ratings__score")).order_by(
            "-avg_rating", "-created_at"
        )
    elif sort == "user":
        recipes = recipes.order_by("uploaded_by__username", "-created_at")
    else:  # date_desc or default
        recipes = recipes.order_by("-created_at")

    if request.headers.get("HX-Request"):
        return render(
            request,
            "partials/recipe_list.html",
            {"recipes": recipes, "user": request.user},
        )

    # Context for filters - only show options relevant to favorited recipes
    uploaders = User.objects.filter(recipes__in=request.user.favorites.all()).distinct()
    tags = Tag.objects.filter(recipes__in=request.user.favorites.all()).distinct()  # ty:ignore[unresolved-attribute]

    return render(
        request,
        "favorites.html",
        {
            "recipes": recipes,
            "version": sandwitches_version,
            "uploaders": uploaders,
            "tags": tags,
            "selected_tags": request.GET.getlist("tag"),  # Add selected_tags
            "user": request.user,
        },
    )


def index(request):
    if not User.objects.filter(is_superuser=True).exists():
        return redirect("setup")

    recipes = Recipe.objects.all().prefetch_related("favorited_by")  # ty:ignore[unresolved-attribute]

    # Only show recipes from people in the admin group
    recipes = recipes.filter(uploaded_by__groups__name="admin")

    # Filtering
    q = request.GET.get("q")
    if q:
        recipes = recipes.filter(
            Q(title__icontains=q) | Q(tags__name__icontains=q)
        ).distinct()

    date_start = request.GET.get("date_start")
    if date_start:
        recipes = recipes.filter(created_at__gte=date_start)

    date_end = request.GET.get("date_end")
    if date_end:
        recipes = recipes.filter(created_at__lte=date_end)

    uploader = request.GET.get("uploader")
    if uploader:
        recipes = recipes.filter(uploaded_by__username=uploader)

    tags = request.GET.getlist("tag")
    if tags:
        recipes = recipes.filter(tags__name__in=tags)

    if request.user.is_authenticated and request.GET.get("favorites") == "on":
        recipes = recipes.filter(pk__in=request.user.favorites.values("pk"))

    # Sorting
    sort = request.GET.get("sort", "date_desc")
    if sort == "date_asc":
        recipes = recipes.order_by("created_at")
    elif sort == "rating":
        recipes = recipes.annotate(avg_rating=Avg("ratings__score")).order_by(
            "-avg_rating", "-created_at"
        )
    elif sort == "user":
        recipes = recipes.order_by("uploaded_by__username", "-created_at")
    else:  # date_desc or default
        recipes = recipes.order_by("-created_at")

    if request.headers.get("HX-Request"):
        return render(
            request,
            "partials/recipe_list.html",
            {"recipes": recipes, "user": request.user},
        )

    # Context for filters
    uploaders = User.objects.filter(recipes__isnull=False).distinct()
    tags = Tag.objects.all()  # ty:ignore[unresolved-attribute]

    highlighted_recipes = Recipe.objects.filter(is_highlighted=True)  # ty:ignore[unresolved-attribute]

    return render(
        request,
        "index.html",
        {
            "recipes": recipes,
            "version": sandwitches_version,
            "uploaders": uploaders,
            "tags": tags,
            "selected_tags": request.GET.getlist("tag"),
            "user": request.user,  # Pass user to template
            "highlighted_recipes": highlighted_recipes,
        },
    )


def setup(request):
    """
    First-time setup page: create initial superuser if none exists.
    Visible only while there are no superusers in the DB.
    """
    from django.contrib.auth.models import Group

    # do not allow access if a superuser already exists
    if User.objects.filter(is_superuser=True).exists():
        return redirect("index")

    if request.method == "POST":
        form = AdminSetupForm(request.POST)
        if form.is_valid():
            user = form.save()

            # Ensure groups exist and add user to admin group
            admin_group, created = Group.objects.get_or_create(name="admin")
            Group.objects.get_or_create(name="community")
            user.groups.add(admin_group)

            user.backend = "django.contrib.auth.backends.ModelBackend"
            login(request, user)
            messages.success(request, _("Admin account created and signed in."))
            return redirect(reverse("admin:index"))
    else:
        form = AdminSetupForm()

    return render(request, "setup.html", {"form": form, "version": sandwitches_version})


def signup(request):
    """
    User signup page: create new regular user accounts.
    """
    from django.contrib.auth.models import Group

    if request.method == "POST":
        form = UserSignupForm(request.POST, request.FILES)
        if form.is_valid():
            user = form.save()

            # Add user to community group
            community_group, created = Group.objects.get_or_create(name="community")
            user.groups.add(community_group)

            # log in the newly created user
            user.backend = "django.contrib.auth.backends.ModelBackend"
            login(request, user)
            messages.success(request, _("Account created and signed in."))
            return redirect("index")
    else:
        form = UserSignupForm()

    return render(
        request, "signup.html", {"form": form, "version": sandwitches_version}
    )


def media(request, file_path=None):
    media_root = getattr(settings, "MEDIA_ROOT", None)
    if not media_root:
        return HttpResponseBadRequest("Invalid Media Root Configuration")
    if not file_path:
        return HttpResponseBadRequest("Invalid File Path")

    base_path = Path(media_root).resolve()
    full_path = base_path.joinpath(file_path).resolve()
    if base_path not in full_path.parents:
        return HttpResponseBadRequest("Access Denied")

    if not full_path.exists() or not full_path.is_file():
        raise Http404("File not found")

    content_type, encoding = mimetypes.guess_type(full_path)
    if not content_type or not content_type.startswith("image/"):
        return HttpResponseBadRequest("Access Denied: Only image files are allowed.")

    response = FileResponse(open(full_path, "rb"), as_attachment=True)
    return response


@login_required
def user_profile(request):
    if request.method == "POST":
        form = UserProfileForm(request.POST, request.FILES, instance=request.user)
        if form.is_valid():
            form.save()
            messages.success(request, _("Profile updated successfully."))
            return redirect("user_profile")
    else:
        form = UserProfileForm(instance=request.user)
    return render(
        request, "profile.html", {"form": form, "version": sandwitches_version}
    )


@login_required
def view_cart(request):
    cart_items = CartItem.objects.filter(user=request.user).select_related("recipe")  # ty:ignore[unresolved-attribute]
    total = sum(item.total_price for item in cart_items)
    return render(
        request,
        "cart.html",
        {
            "cart_items": cart_items,
            "total": total,
            "version": sandwitches_version,
        },
    )


@login_required
def add_to_cart(request, pk):
    recipe = get_object_or_404(Recipe, pk=pk)
    if not recipe.price:
        messages.error(request, _("This recipe cannot be ordered (no price set)."))
        return redirect("recipe_detail", slug=recipe.slug)

    cart_item, created = CartItem.objects.get_or_create(  # ty:ignore[unresolved-attribute]
        user=request.user, recipe=recipe
    )
    if not created:
        cart_item.quantity += 1
        cart_item.save()

    messages.success(
        request, _("Added %(title)s to your cart.") % {"title": recipe.title}
    )
    return redirect("view_cart")


@login_required
def remove_from_cart(request, pk):
    cart_item = get_object_or_404(CartItem, pk=pk, user=request.user)
    cart_item.delete()
    messages.success(request, _("Removed from cart."))
    return redirect("view_cart")


@login_required
def update_cart_quantity(request, pk):
    if request.method == "POST":
        cart_item = get_object_or_404(CartItem, pk=pk, user=request.user)
        try:
            quantity = int(request.POST.get("quantity", 1))
            if quantity > 0:
                cart_item.quantity = quantity
                cart_item.save()
            else:
                cart_item.delete()
        except ValueError:
            pass
    return redirect("view_cart")


@login_required
def checkout_cart(request):
    cart_items = CartItem.objects.filter(user=request.user)  # ty:ignore[unresolved-attribute]
    if not cart_items.exists():
        messages.error(request, _("Your cart is empty."))
        return redirect("view_cart")

    created_orders = []  # noqa: F841
    errors = []

    # We use a transaction to ensure either all orders are created or none if something goes wrong
    from django.db import transaction

    try:
        with transaction.atomic():
            for item in cart_items:
                # Create Order for each recipe in cart (quantity times?)
                # Current Order model doesn't have quantity, so we create multiple orders or update Order model.
                # For now, let's create 'quantity' number of orders as per current schema
                # OR we could update Order model to support quantity.
                # Let's see if Order has quantity. (Checked: it does not).
                for i in range(item.quantity):
                    try:
                        Order.objects.create(user=request.user, recipe=item.recipe)  # ty:ignore[unresolved-attribute]
                    except (ValidationError, ValueError) as e:
                        errors.append(f"{item.recipe.title}: {str(e)}")
                        raise e  # Trigger rollback

            cart_items.delete()
            messages.success(request, _("Orders submitted successfully!"))
            return redirect("user_profile")
    except Exception:
        if errors:
            for error in errors:
                messages.error(request, error)
        else:
            messages.error(request, _("An error occurred during checkout."))
        return redirect("view_cart")



# Generated by Django 6.0.1 on 2026-01-17 16:48

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0005_rating_comment"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalrecipe",
            name="is_highlighted",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="recipe",
            name="is_highlighted",
            field=models.BooleanField(default=False),
        ),
    ]



from django.db import migrations


def create_groups(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    Group.objects.get_or_create(name="admin")
    Group.objects.get_or_create(name="community")


def remove_groups(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    Group.objects.filter(name__in=["admin", "community"]).delete()


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0013_cartitem"),
    ]

    operations = [
        migrations.RunPython(create_groups, remove_groups),
    ]



# from gunicorn.http.wsgi import log
import logging

# from django.core.mail import send_mail
from django_tasks import task
from django.contrib.auth import get_user_model

from django.core.mail import EmailMultiAlternatives
from django.conf import settings
from django.utils.translation import gettext as _


import textwrap


@task(takes_context=True, priority=2, queue_name="emails")
def email_users(context, recipe_id):
    logging.debug(
        f"Attempt {context.attempt} to send users an email. Task result id: {context.task_result.id}."
    )

    User = get_user_model()
    emails = list(
        User.objects.exclude(email__isnull=True)
        .exclude(email="")
        .values_list("email", flat=True)
    )

    if not emails:
        logging.warning("No users with valid emails found.")
        return 0

    send_emails(recipe_id, emails)

    return True


@task(priority=5)
def reset_daily_orders():
    from .models import Recipe

    count = Recipe.objects.update(daily_orders_count=0)  # ty:ignore[unresolved-attribute]
    logging.info(f"Successfully reset daily order count for {count} recipes.")
    return count


@task(priority=2, queue_name="emails")
def notify_order_submitted(order_id):
    from .models import Order

    try:
        order = Order.objects.select_related("user", "recipe").get(pk=order_id)  # ty:ignore[unresolved-attribute]
    except Order.DoesNotExist:  # ty:ignore[unresolved-attribute]
        logging.warning(f"Order {order_id} not found. Skipping notification.")
        return

    user = order.user
    if not user.email:
        logging.warning(f"User {user.username} has no email. Skipping notification.")
        return

    recipe = order.recipe
    subject = _("Order Confirmation: %(recipe_title)s") % {"recipe_title": recipe.title}
    from_email = getattr(settings, "EMAIL_FROM_ADDRESS")

    context_data = {
        "user_name": user.get_full_name() or user.username,
        "recipe_title": recipe.title,
        "order_id": order.id,
        "total_price": order.total_price,
    }

    text_content = (
        _(
            "Hello %(user_name)s,\n\n"
            "Your order for %(recipe_title)s has been successfully submitted!\n"
            "Order ID: %(order_id)s\n"
            "Total Price: %(total_price)s\n\n"
            "Thank you for ordering with Sandwitches.\n"
        )
        % context_data
    )

    html_content = (
        _(
            "<div style='font-family: sans-serif;'>"
            "<h2>Order Confirmation</h2>"
            "<p>Hello <strong>%(user_name)s</strong>,</p>"
            "<p>Your order for <strong>%(recipe_title)s</strong> has been successfully submitted!</p>"
            "<ul>"
            "<li>Order ID: %(order_id)s</li>"
            "<li>Total Price: %(total_price)s</li>"
            "</ul>"
            "<p>Thank you for ordering with Sandwitches.</p>"
            "</div>"
        )
        % context_data
    )

    msg = EmailMultiAlternatives(
        subject=subject,
        body=text_content,
        from_email=from_email,
        to=[user.email],
    )
    msg.attach_alternative(html_content, "text/html")
    msg.send()

    logging.info(f"Order confirmation email sent to {user.email} for order {order.id}")


def send_emails(recipe_id, emails):
    from .models import Recipe

    logging.debug(f"Preparing to send email to: {emails}")
    recipe = Recipe.objects.get(pk=recipe_id)  # ty:ignore[unresolved-attribute]
    from_email = getattr(settings, "EMAIL_FROM_ADDRESS")

    recipe_slug = recipe.get_absolute_url()
    base_url = (
        settings.CSRF_TRUSTED_ORIGINS[0]
        if settings.CSRF_TRUSTED_ORIGINS
        else "http://localhost"
    ).rstrip("/")

    raw_message_fmt = _("""
    Hungry? We just added <strong>%(title)s</strong> to our collection.

    It's a delicious recipe that you won't want to miss!
    %(description)s

    Check out the full recipe, ingredients, and steps here:
    %(url)s

    Happy Cooking!

    The Sandwitches Team
    """)

    context_data = {
        "title": recipe.title,
        "uploaded_by": recipe.uploaded_by,
        "description": recipe.description,
        "url": f"{base_url}{recipe_slug}",
        "image_url": f"{base_url}{recipe.image.url}" if recipe.image else "",
    }

    wrapped_message = textwrap.fill(
        textwrap.dedent(raw_message_fmt) % context_data, width=70
    )

    html_content_fmt = _("""
    <div style="font-family: 'Helvetica', sans-serif; max-width: 600px; margin: auto; border: 1px solid #eee; padding: 20px;">
        <h2 style="color: #d35400; text-align: center;">New Recipe: %(title)s by %(uploaded_by)s</h2>
        <div style="text-align: center; margin: 20px 0;">
            <img src="%(image_url)s" alt="%(title)s" style="width: 100%%; border-radius: 8px;">
        </div>
        <p style="font-size: 16px; line-height: 1.5; color: #333;">
            Hungry? We just added <strong>%(title)s</strong> to our collection.
            <br>
            It's a delicious recipe that you won't want to miss!
            <br>
            %(description)s
            <br>
            Check out the full recipe, ingredients, and steps here:
            Click the button below to see how to make it!
            <br>
            Happy Cooking!
            <br>
            The Sandwitches Team
        </p>
        <div style="text-align: center; margin-top: 30px;">
            <a href="%(url)s" style="background-color: #e67e22; color: white; padding: 12px 25px; text-decoration: none; border-radius: 5px; font-weight: bold;">VIEW RECIPE</a>
        </div>
    </div>
    """)

    html_content = html_content_fmt % context_data

    subject = _("Sandwitches - New Recipe: %(title)s by %(uploaded_by)s") % context_data
    for email in emails:
        msg = EmailMultiAlternatives(
            subject=subject,
            body=wrapped_message,
            from_email=from_email,
            to=[email],
        )
        msg.attach_alternative(html_content, "text/html")
        msg.send()



# Generated by Django 6.0.1 on 2026-01-21 09:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0007_historicalrecipe_price_recipe_price_order"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalrecipe",
            name="daily_orders_count",
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name="historicalrecipe",
            name="max_daily_orders",
            field=models.PositiveIntegerField(
                blank=True, null=True, verbose_name="Max daily orders"
            ),
        ),
        migrations.AddField(
            model_name="recipe",
            name="daily_orders_count",
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name="recipe",
            name="max_daily_orders",
            field=models.PositiveIntegerField(
                blank=True, null=True, verbose_name="Max daily orders"
            ),
        ),
    ]



# Generated by Django 6.0 on 2026-01-13 11:59

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalrecipe",
            name="servings",
            field=models.IntegerField(
                default=1, validators=[django.core.validators.MinValueValidator(1)]
            ),
        ),
        migrations.AddField(
            model_name="recipe",
            name="servings",
            field=models.IntegerField(
                default=1, validators=[django.core.validators.MinValueValidator(1)]
            ),
        ),
    ]



from django import template

register = template.Library()


@register.filter
def split(value, arg):
    return value.split(arg)


@register.filter
def strip(value):
    if isinstance(value, str):
        return value.strip()
    return value



# Generated by Django 6.0.1 on 2026-01-25 11:12

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "sandwitches",
            "0012_rename_is_community_made_historicalrecipe_is_approved_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="CartItem",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("quantity", models.PositiveIntegerField(default=1)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "recipe",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cart_items",
                        to="sandwitches.recipe",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cart_items",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Cart Item",
                "verbose_name_plural": "Cart Items",
                "unique_together": {("user", "recipe")},
            },
        ),
    ]



# Generated by Django 6.0 on 2026-01-14 09:13

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0002_historicalrecipe_servings_recipe_servings"),
    ]

    operations = [
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("site_name", models.CharField(default="Sandwitches", max_length=255)),
                ("site_description", models.TextField(blank=True)),
                ("email", models.EmailField(blank=True, max_length=254)),
                ("ai_connection_point", models.URLField(blank=True)),
                ("ai_model", models.CharField(blank=True, max_length=255)),
                ("ai_api_key", models.CharField(blank=True, max_length=255)),
            ],
            options={
                "verbose_name": "Site Settings",
            },
        ),
    ]



from django.contrib.syndication.views import Feed
from django.urls import reverse_lazy
from .models import Recipe


class LatestRecipesFeed(Feed):
    title = "Sandwitches - Latest Recipes"
    link = reverse_lazy(
        "index"
    )  # This should point to the homepage or a list of recipes
    description = "Updates on the newest recipes added to Sandwitches."

    def items(self):
        return Recipe.objects.order_by("-created_at")[:5]  # ty:ignore[unresolved-attribute]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.description

    def item_link(self, item):
        return item.get_absolute_url()



# Generated by Django 6.0.1 on 2026-01-24 15:32

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0009_historicalrecipe_is_approved_recipe_is_approved"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalrecipe",
            old_name="is_approved",
            new_name="is_community_made",
        ),
        migrations.RenameField(
            model_name="recipe",
            old_name="is_approved",
            new_name="is_community_made",
        ),
    ]



"""
URL configuration for sandwitches project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.contrib import admin
from django.urls import path, include
from . import views
from .api import api
from django.conf.urls.i18n import i18n_patterns
from .feeds import LatestRecipesFeed  # Import the feed class
from django.contrib.auth.views import LogoutView  # Import LogoutView


import os
import sys


urlpatterns = [
    path("i18n/", include("django.conf.urls.i18n")),
    path("signup/", views.signup, name="signup"),
    path("login/", views.CustomLoginView.as_view(), name="login"),
    path("logout/", LogoutView.as_view(next_page="index"), name="logout"),
    path("profile/", views.user_profile, name="user_profile"),
    path("community/", views.community, name="community"),
    path("admin/", admin.site.urls),
    path("api/", api.urls),
    path("media/<path:file_path>", views.media, name="media"),
    path("favorites/", views.favorites, name="favorites"),
    path("cart/", views.view_cart, name="view_cart"),
    path("cart/add/<int:pk>/", views.add_to_cart, name="add_to_cart"),
    path("cart/remove/<int:pk>/", views.remove_from_cart, name="remove_from_cart"),
    path(
        "cart/update/<int:pk>/", views.update_cart_quantity, name="update_cart_quantity"
    ),
    path("cart/checkout/", views.checkout_cart, name="checkout_cart"),
    path("", views.index, name="index"),
    path("feeds/latest/", LatestRecipesFeed(), name="latest_recipes_feed"),
    path(
        "feeds/latest/", LatestRecipesFeed(), name="latest_recipes_feed"
    ),  # Add this line
]

urlpatterns += i18n_patterns(
    path("recipes/<slug:slug>/", views.recipe_detail, name="recipe_detail"),
    path("setup/", views.setup, name="setup"),
    path("recipes/<int:pk>/rate/", views.recipe_rate, name="recipe_rate"),
    path("recipes/<int:pk>/order/", views.order_recipe, name="order_recipe"),
    path("recipes/<int:pk>/favorite/", views.toggle_favorite, name="toggle_favorite"),
    path("dashboard/", views.admin_dashboard, name="admin_dashboard"),
    path("dashboard/recipes/", views.admin_recipe_list, name="admin_recipe_list"),
    path(
        "dashboard/approvals/",
        views.admin_recipe_approval_list,
        name="admin_recipe_approval_list",
    ),
    path("dashboard/recipes/add/", views.admin_recipe_add, name="admin_recipe_add"),
    path(
        "dashboard/recipes/<int:pk>/edit/",
        views.admin_recipe_edit,
        name="admin_recipe_edit",
    ),
    path(
        "dashboard/recipes/<int:pk>/delete/",
        views.admin_recipe_delete,
        name="admin_recipe_delete",
    ),
    path(
        "dashboard/recipes/<int:pk>/approve/",
        views.admin_recipe_approve,
        name="admin_recipe_approve",
    ),
    path(
        "dashboard/recipes/<int:pk>/rotate/",
        views.admin_recipe_rotate,
        name="admin_recipe_rotate",
    ),
    path("dashboard/users/", views.admin_user_list, name="admin_user_list"),
    path(
        "dashboard/users/<int:pk>/edit/", views.admin_user_edit, name="admin_user_edit"
    ),
    path(
        "dashboard/users/<int:pk>/delete/",
        views.admin_user_delete,
        name="admin_user_delete",
    ),
    path("dashboard/tags/", views.admin_tag_list, name="admin_tag_list"),
    path("dashboard/tags/add/", views.admin_tag_add, name="admin_tag_add"),
    path(
        "dashboard/tags/<int:pk>/edit/",
        views.admin_tag_edit,
        name="admin_tag_edit",
    ),
    path(
        "dashboard/tags/<int:pk>/delete/",
        views.admin_tag_delete,
        name="admin_tag_delete",
    ),
    path("dashboard/tasks/", views.admin_task_list, name="admin_task_list"),
    path(
        "dashboard/tasks/<str:pk>/", views.admin_task_detail, name="admin_task_detail"
    ),
    path("dashboard/ratings/", views.admin_rating_list, name="admin_rating_list"),
    path(
        "dashboard/ratings/<int:pk>/delete/",
        views.admin_rating_delete,
        name="admin_rating_delete",
    ),
    path("dashboard/orders/", views.admin_order_list, name="admin_order_list"),
    prefix_default_language=True,
)

if "test" not in sys.argv or "PYTEST_VERSION" in os.environ:
    from debug_toolbar.toolbar import debug_toolbar_urls

    urlpatterns = [
        *urlpatterns,
    ] + debug_toolbar_urls()



# Generated by Django 6.0 on 2026-01-14 10:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sandwitches", "0004_alter_setting_ai_api_key_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="rating",
            name="comment",
            field=models.TextField(blank=True),
        ),
    ]



from django.core.management.base import BaseCommand
from sandwitches.tasks import reset_daily_orders


class Command(BaseCommand):
    help = "Resets the daily order count for all recipes. Should be run at midnight."

    def handle(self, *args, **options):
        task_result = reset_daily_orders.enqueue()
        self.stdout.write(
            self.style.SUCCESS(
                f"Enqueued daily order count reset task (Result ID: {task_result.id})."
            )
        )



import hashlib
import os
from django.core.files.base import ContentFile
from django.core.files.storage import FileSystemStorage
from pathlib import Path
from django.conf import settings
import logging


class HashedFilenameStorage(FileSystemStorage):
    """
    Save uploaded files under a hash of their contents + original extension.
    Example output: media/recipes/3f8a9d...png
    """

    def _save(self, name, content):
        try:
            content.seek(0)
        except Exception:
            pass

        data = content.read()
        h = hashlib.sha256(data).hexdigest()[:32]
        ext = os.path.splitext(name)[1].lower() or ""
        name = f"recipes/{h}{ext}"

        content = ContentFile(data)
        return super()._save(name, content)


def is_database_readable(path=None) -> bool:
    """
    Check whether the database file exists and is readable.

    If `path` is None, uses `django.conf.settings.DATABASE_FILE` by default.

    Returns:
        bool: True if the file exists and is readable by the current process, False otherwise.
    """

    if path is None:
        path = getattr(settings, "DATABASE_FILE", None)

    if not path:
        return False

    p = Path(path)
    logging.debug(f"Checking database file readability at: {p}")
    try:
        if p.is_file():
            with open(p, "r"):  # Removed 'as f'
                # If we can open it, it's readable. No need to read content.
                pass
            logging.debug(f"Database file at {p} is readable.")
            return True
        else:
            logging.error(f"Database file at {p} does not exist.")
            return False
    except IOError:
        logging.error(
            f"Database file at {p} is not readable due to permission or other IO error."
        )
        return False


def is_database_writable(path=None) -> bool:
    """
    Check whether the database file exists and is writable.

    If `path` is None, uses `django.conf.settings.DATABASE_FILE` by default.

    Returns:
        bool: True if the file exists and is writable by the current process, False otherwise.
    """

    if path is None:
        path = getattr(settings, "DATABASE_FILE", None)

    if not path:
        return False

    p = Path(path)
    logging.debug(f"Checking database file writability at: {p}")
    try:
        # If the file exists, try to open it for appending
        if p.is_file():
            with open(p, "a"):  # Removed 'as f'
                pass
            logging.debug(f"Database file at {p} is writable.")
            return True
        else:
            # If file does not exist, check if its parent directory is writable
            if p.parent.is_dir() and os.access(p.parent, os.W_OK):
                # Try creating a dummy file to confirm writability
                dummy_file = p.parent / f".tmp_writable_test_{os.getpid()}"
                try:
                    dummy_file.touch()
                    dummy_file.unlink()
                    logging.debug(f"Database path at {p.parent} is writable.")
                    return True
                except IOError:
                    logging.error(f"Cannot create dummy file in {p.parent}.")
                    return False
            else:
                logging.error(
                    f"Parent directory {p.parent} is not writable or does not exist."
                )
                return False

    except IOError:
        logging.error(
            f"Database file at {p} is not writable due to permission or other IO error."
        )
        return False



from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm
from django.utils.translation import gettext_lazy as _
from .models import Recipe, Tag, Setting

User = get_user_model()


class BaseUserFormMixin:
    """Mixin to handle common password validation and user field processing."""

    def clean_passwords(self, cleaned_data):
        p1 = cleaned_data.get("password1")
        p2 = cleaned_data.get("password2")
        if p1 and p2 and p1 != p2:
            raise forms.ValidationError(_("Passwords do not match."))
        return cleaned_data

    def _set_user_attributes(self, user, data):
        """Helper to apply optional name fields."""
        user.first_name = data.get("first_name", "")
        user.last_name = data.get("last_name", "")
        user.save()
        return user


class AdminSetupForm(forms.ModelForm, BaseUserFormMixin):
    password1 = forms.CharField(widget=forms.PasswordInput, label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput, label=_("Confirm Password"))

    class Meta:
        model = User
        fields = ("username", "first_name", "last_name", "email")

    def clean(self):
        cleaned_data = super().clean()
        return self.clean_passwords(cleaned_data)

    def save(self, commit=True):
        data = self.cleaned_data
        user = User.objects.create_superuser(
            username=data["username"], email=data["email"], password=data["password1"]
        )
        return self._set_user_attributes(user, data)


class UserSignupForm(UserCreationForm, BaseUserFormMixin):
    language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        label=_("Preferred language"),
        initial=settings.LANGUAGE_CODE,
    )
    avatar = forms.ImageField(label=_("Profile Image"), required=False)
    bio = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 3}), label=_("Bio"), required=False
    )

    class Meta(UserCreationForm.Meta):
        model = User
        fields = (
            "username",
            "first_name",
            "last_name",
            "email",
            "language",
            "avatar",
            "bio",
        )

    def clean(self):
        return super().clean()

    def save(self, commit=True):
        user = super().save(commit=False)
        user.is_superuser = False
        user.is_staff = False
        # Explicitly save the extra fields if they aren't automatically handled by ModelForm save (they should be if in Meta.fields)
        user.language = self.cleaned_data["language"]
        user.avatar = self.cleaned_data["avatar"]
        user.bio = self.cleaned_data["bio"]
        if commit:
            user.save()
        return user


class UserProfileForm(forms.ModelForm):
    class Meta:
        model = User
        fields = (
            "first_name",
            "last_name",
            "email",
            "avatar",
            "bio",
        )


class UserEditForm(forms.ModelForm):
    class Meta:
        model = User
        fields = (
            "username",
            "first_name",
            "last_name",
            "email",
            "is_staff",
            "is_active",
            "language",
            "avatar",
            "bio",
        )


class TagForm(forms.ModelForm):
    class Meta:
        model = Tag
        fields = ("name",)


class RecipeForm(forms.ModelForm):
    tags_string = forms.CharField(
        required=False,
        label=_("Tags (comma separated)"),
        widget=forms.TextInput(attrs={"placeholder": _("e.g. spicy, vegan, quick")}),
    )
    rotation = forms.IntegerField(widget=forms.HiddenInput(), initial=0, required=False)

    class Meta:
        model = Recipe
        fields = [
            "title",
            "image",
            "uploaded_by",
            "description",
            "ingredients",
            "instructions",
            "price",
            "is_highlighted",
            "is_approved",
            "max_daily_orders",
        ]
        widgets = {
            "image": forms.FileInput(),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields["tags_string"].initial = ", ".join(
                self.instance.tags.values_list("name", flat=True)
            )

    def save(self, commit=True):
        recipe = super().save(commit=commit)

        # Handle rotation if an image exists and rotation is requested
        rotation = self.cleaned_data.get("rotation", 0)
        if rotation != 0 and recipe.image:
            try:
                from PIL import Image as PILImage

                img = PILImage.open(recipe.image.path)
                # PIL rotates counter-clockwise by default, our 'rotation' is clockwise
                img = img.rotate(-rotation, expand=True)
                img.save(recipe.image.path)
            except Exception as e:
                print(f"Error rotating image: {e}")

        if commit:
            recipe.set_tags_from_string(self.cleaned_data.get("tags_string", ""))
        else:
            # We'll need to handle this in the view if commit=False
            self.save_m2m = lambda: recipe.set_tags_from_string(
                self.cleaned_data.get("tags_string", "")
            )
        return recipe


class UserRecipeSubmissionForm(forms.ModelForm):
    tags_string = forms.CharField(
        required=False,
        label=_("Tags (comma separated)"),
        widget=forms.TextInput(attrs={"placeholder": _("e.g. spicy, vegan, quick")}),
    )

    class Meta:
        model = Recipe
        fields = [
            "title",
            "image",
            "description",
            "ingredients",
            "instructions",
            "price",
            "servings",
        ]
        widgets = {
            "image": forms.FileInput(),
        }

    def save(self, commit=True):
        recipe = super().save(commit=commit)
        if commit:
            recipe.set_tags_from_string(self.cleaned_data.get("tags_string", ""))
        else:
            self.save_m2m = lambda: recipe.set_tags_from_string(
                self.cleaned_data.get("tags_string", "")
            )
        return recipe


class RatingForm(forms.Form):
    """Form for rating recipes (0-10) with an optional comment."""

    score = forms.FloatField(
        min_value=0.0,
        max_value=10.0,
        widget=forms.NumberInput(
            attrs={"step": "0.1", "min": "0", "max": "10", "class": "slider"}
        ),
        label=_("Your rating"),
    )
    comment = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 2}),
        label=_("Comment (optional)"),
        required=False,
    )


class SettingForm(forms.ModelForm):
    class Meta:
        model = Setting
        fields = [
            "site_name",
            "site_description",
            "email",
            "ai_connection_point",
            "ai_model",
            "ai_api_key",
        ]
        widgets = {
            "ai_api_key": forms.PasswordInput(render_value=True),
        }



"""
Django settings for sandwitches project.

Generated by 'django-admin startproject' using Django 5.2.7.

For more information on this file, see
https://docs.djangoproject.com/en/5.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.2/ref/settings/
"""

from pathlib import Path
import os

from django.core.exceptions import ImproperlyConfigured
from . import storage

DEBUG = bool(os.environ.get("DEBUG", default=0))

SECRET_KEY = os.environ.get("SECRET_KEY")
if not SECRET_KEY:
    raise ImproperlyConfigured(
        "The SECRET_KEY environment variable must be set in production."
    )

ALLOWED_HOSTS = os.environ.get("ALLOWED_HOSTS", "127.0.0.1, localhost").split(",")
CSRF_TRUSTED_ORIGINS = os.environ.get("CSRF_TRUSTED_ORIGINS", "").split(",")
DATABASE_FILE = Path(os.environ.get("DATABASE_FILE", default="/db/db.sqlite3"))

storage.is_database_readable(DATABASE_FILE)
storage.is_database_writable(DATABASE_FILE)


# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


TASKS = {
    "default": {
        "BACKEND": "django_tasks.backends.database.DatabaseBackend",
        "QUEUES": ["default", "emails"],
        "OPTIONS": {
            "queues": {
                "low_priority": {
                    "max_attempts": 5,
                }
            },
            "max_attempts": 10,
            "backoff_factor": 3,
            "purge": {"finished": "10 days", "unfinished": "20 days"},
        },
    }
}

# Application definition
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "sandwitches",
    "django_tasks",
    "django_tasks.backends.database",
    "debug_toolbar",
    "imagekit",
    "import_export",
    "simple_history",
    "solo",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "debug_toolbar.middleware.DebugToolbarMiddleware",
    "simple_history.middleware.HistoryRequestMiddleware",
]

ROOT_URLCONF = "sandwitches.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.csrf",
                "django.template.context_processors.i18n",
            ],
        },
    },
]

WSGI_APPLICATION = "sandwitches.wsgi.application"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": DATABASE_FILE,
    }
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": os.getenv("LOG_LEVEL", "INFO"),
            "propagate": False,
        },
    },
}

LOGIN_REDIRECT_URL = "index"
LOGOUT_REDIRECT_URL = "index"
LOGIN_URL = "login"


# Password validation
# https://docs.djangoproject.com/en/5.2/ref/settings/#auth-password-validators

AUTH_USER_MODEL = "sandwitches.User"

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Media files (for uploaded images)
MEDIA_URL = "/media/"
MEDIA_ROOT = Path(os.environ.get("MEDIA_ROOT", default=BASE_DIR / "media"))

# Static (for CSS etc)
STATIC_URL = "/static/"
STATIC_ROOT = Path("/tmp/staticfiles")
STATICFILES_DIRS = [BASE_DIR / "static", MEDIA_ROOT]

LANGUAGE_CODE = "en"
TIME_ZONE = "UTC"
USE_I18N = True
LANGUAGES = [
    ("en", "English"),
    ("nl", "Nederlands"),
]

LOCALE_PATHS = [BASE_DIR / "locale"]

USE_TZ = True

# EU Date formats
DATE_FORMAT = "d/m/Y"
DATETIME_FORMAT = "d/m/Y H:i:s"
SHORT_DATE_FORMAT = "d/m/Y"
SHORT_DATETIME_FORMAT = "d/m/Y H:i"

INTERNAL_IPS = [
    "127.0.0.1",
]

STORAGES = {
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    },
    "staticfiles": {
        "BACKEND": "whitenoise.storage.CompressedManifestStaticFilesStorage",
    },
}


EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
EMAIL_USE_TLS = os.environ.get("SMTP_USE_TLS")
EMAIL_HOST = os.environ.get("SMTP_HOST")
EMAIL_HOST_USER = os.environ.get("SMTP_USER")
EMAIL_HOST_PASSWORD = os.environ.get("SMTP_PASSWORD")
EMAIL_PORT = os.environ.get("SMTP_PORT")
EMAIL_FROM_ADDRESS = os.environ.get("SMTP_FROM_EMAIL")
SEND_EMAIL = all(
    v is not None
    for v in [
        EMAIL_HOST,
        EMAIL_HOST_USER,
        EMAIL_HOST_PASSWORD,
        EMAIL_PORT,
        EMAIL_FROM_ADDRESS,
    ]
)

# Default primary key field type
# https://docs.djangoproject.com/en/5.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
